@@ -295,3 +295,11 @@ type DependencyCycleError []string
 func (err DependencyCycleError) Error() string {
 	return "Found a dependency cycle between modules: " + strings.Join([]string(err), " -> ")
 }
+
+type DependencyInvalidConfigPathError struct {
+	DependencyName string
+}
+
+func (err DependencyInvalidConfigPathError) Error() string {
+	return fmt.Sprintf("dependency %q has invalid config_path", err.DependencyName)
+}
